// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

var Cc = Components.classes;
var Ci = Components.interfaces;

var gB;
var gEditor = null;

var gBlogService = Cc['@flock.com/flock-blog;1'].getService(Ci['flockIBlogService']);

/*
 * Coop stuff
 */

var faves_coop = Cc['@flock.com/singleton;1'].getService(Ci.flockISingleton).getSingleton('chrome://flock/content/common/load-faves-coop.js').wrappedJSObject;

/*
 * Helpers
 */

function xmlEscape(aString) {
  var result;
  result = aString.replace(/&/, "&amp;");
  result = result.replace(/</, "&lt;");
  result = result.replace(/>/, "&gt;");
  result = result.replace(/"/, "&quot;");
  return result;
}

// nsIStringEnumerator implementation
function stringEnumerator (aArray) {
  aArray.hasMore = function () { 
    return this.length != 0; 
  }
  aArray.getNext = function () { 
    var rval = this.shift (); 
    dump (rval+"\n");
    return rval;
  }
  return aArray;
}

function populatesBlogList() {
  var blogList = $("flock-blog-accounts-popup");
  var accounts = faves_coop.get('http://flock.com/rdf#AccountsRoot');
  var enum = accounts.children.enumerate();
  while (enum.hasMoreElements()) {
    var account = enum.getNext();
    var enum2 = account.children.enumerate();
    while (enum2.hasMoreElements()) {
      var child = enum2.getNext();
      if (child.isInstanceOf(faves_coop.Blog)) {
        var item = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
        item.setAttribute("class", "menuitem-iconic");
        item.setAttribute("id", child.id());
        item.setAttribute("label", child.title);
        item.setAttribute("src", account.favicon);
        item.setAttribute("oncommand", "setupBlog('"+child.id().replace(/'/g, "\\'")+"');");
        dump("We just set up "+item.getAttribute('oncommand')+"\n");
        blogList.appendChild(item);
      }
    }
  }

  var accts = $("blog-account-selector");
  var defaultBlogId = gBlogService.getDefaultBlog();
  if (defaultBlogId == "") {
    accts.selectedIndex = 0;
    defaultBlogId = accts.selectedItem.id;
    gBlogService.setDefaultBlog(defaultBlogId);
  }
  accts.selectedItem = $(defaultBlogId);
  setupBlog(defaultBlogId);
}

function setupBlog(aId)
{
  gB = faves_coop.get(aId);
  try {
    gWebService = Cc[gB.getParent().serviceId].getService(Ci.flockIBlogWebService);
  } catch(e) {
    dump("*** ERROR: Couldn't create the component: "+gB.getParent().serviceId+"\n");
  }
  
  // Enable categories if supported
  switch (gWebService.supportsCategories) {
    case 0:
      $('flock-blog-categories-container').setAttribute('collapsed', true);
      break;
    case 1:
      $('flock-blog-categories-container').setAttribute('collapsed', false);
      $('flock-blog-categories').setAttribute('seltype', 'single');
      break;
    case 2:
      $('flock-blog-categories-container').setAttribute('collapsed', false);
      $('flock-blog-categories').setAttribute('seltype', 'multiple');
      break;
  }
  if (gWebService.metadataOverlay && gWebService.metadataOverlay != "") {
    dump(gWebService.metadataOverlay);
    $('extra-attributes').setAttribute('src', gWebService.metadataOverlay);
    $('extra-attributes').setAttribute('collapsed', 'false');
    // set the right size
  }
  else {
    $('extra-attributes').setAttribute('collapsed', 'true');
  }

  flock_blogEditorSetupCategories();
  if ((!refreshCategories.inProgress) && (gWebService.supportsCategories > 0))
    refreshCategories.exec();
}



/*
 * load/close functions
 */

editorOnload = function() {
  var title = "";
  var content = "";
  var tags = "";

  gEditor = new editorWidget($('editordeck'), $('edit'), $('shelfEditorToolbar'));
  populatesBlogList();
  

  flock_blogEditorSetupEditorCommands();
};


/*
 * Toolbar customization
 */

function BrowserCustomizeToolbar()
{
  window.openDialog("chrome://flock/content/blog/customizeToolbar.xul", "CustomizeToolbar",
    "chrome,all,dependent", document.getElementById("shelfEditorToolbox"));
}

/*
 * Metadata
 */

function flock_blogEditorSetupCategories() {
  var catList = $('flock-blog-categories');

  var enum = gB.categories.enumerate();
  while (catList.getRowCount() > 0)
    catList.removeItemAt(0);
  while (enum.hasMoreElements()) {
    var category = enum.getNext();
    catList.appendItem(category.name, category.categoryId);
  }
}

var refreshCategories = {
  exec: function() {
    this.inProgress = true;
    $("categories-throbber").setAttribute("hidden", "false");
    gWebService.getCategoryList(this, gB.id());
  },
  // Implement refresh listener
  onFault: function (aFault) {
    this.inProgress = false;
    $("categories-throbber").setAttribute("hidden", "true");
  },
  onResult: function (aResult) {
    //dump_rdf(aResult);
    var svc = Cc['@flock.com/flock-blog;1'].getService(Ci.flockIBlogService);
    while (aResult.hasMoreElements()) {
      category = aResult.getNext().QueryInterface(Ci.flockIBlogCategory);
      dump("Add category...\n");
      svc.addCategory(gB.id(), category.id, category.label);
    }
    flock_blogEditorSetupCategories();
    this.inProgress = false;
    $("categories-throbber").setAttribute("hidden", "true");
  },

  onError: function (aFault) {
    debugger;
    var strbundle = document.getElementById("blogStringBundle");
    alert(strbundle.getString("flock.blog.metadata.error.refreshingCategories"));
    this.inProgress = false;
    $("categories-throbber").setAttribute("hidden", "true");
  },

  inProgress: false,
}

function getExtraParameters() {
  var result = new Array();

  var extra = $("extra-attributes").contentDocument.getElementById('extraData');
  if (!extra)
    return result;
  var rowsTag = extra.getElementsByTagName('rows')[0];
  var rows = rowsTag.getElementsByTagName('row');
  for (i=0; i<rows.length; i++) {
    dump("Found parameter: "+rows[i].getAttribute('value')+"\n");
    result.push(rows[i].getAttribute('value'));
  }
  return stringEnumerator(result);
}

function getCategoryIdArray() {
  var rval = new Array();
  var categories = $("flock-blog-categories").selectedItems;
  for(var i=0; i<categories.length; ++i) {
    var category = categories[i];
    rval.push(category.getAttribute("value"));
  }
  return stringEnumerator(rval);
}



/*
 * Commands 
 */

var gBlogEditCommandController = null;

var flock_blogLink = {
  isEnabled: function() {
    return gEditor.linkEnabled();
  },
  exec: function() {
    gEditor.link();
  }
}

var flock_blogUnlink = {
  isEnabled: function() {
    return gEditor.unlinkEnabled();
  },
  exec: function() {
    gEditor.execCommand('unlink');
  }
}

function publish() {
  // Setup an account if needed
  if(gBlogService.accountCount() == 0)
    flock_blogOpenWizard();

  // If the user don't setup a blog, don't continue
  if(gBlogService.accountCount() == 0)
    return;

  // Get the xhtml source (from the editor or the source pane)
  var html;
  if ($('editordeck').selectedIndex == 0)
    html = gEditor.getXHtml();
  else
    html = $('editsource').value;
  html = html.replace(/\n/g, "<br/>");
 
  var post = {};
  post.getInterfaces = function (count) {
    var interfaceList = [Ci['flockIBlogPost'], nsIClassInfo];
    count.value = interfaceList.length;
    return interfaceList;
  };
  post.QueryInterface = function (iid) {
    if (!iid.equals(Ci.flockIBlogPost))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
  };
 var title = $('flock-blog-title').value;
  post.title = xmlEscape(title);
  post.description = html;
  var tags = $("flock-blog-tags").value.replace(' ', ''); 
  if (tags == '') 
  post.tags = stringEnumerator([]); 
  else  
  post.tags = stringEnumerator(tags.split(','));   
  post.categories = getCategoryIdArray();
  post.extra = getExtraParameters();

  var blog = faves_coop.get($("blog-account-selector").selectedItem.id);

  var listener = {
    onResult: function(aResult) {
      $("accept-button").setAttribute('disabled', 'false');
      var wm = Cc['@mozilla.org/appshell/window-mediator;1'].getService(Ci.nsIWindowMediator);
      recent = wm.getMostRecentWindow("navigator:browser");
      var linkprefs = Cc["@mozilla.org/preferences-service;1"]
            .getService(Ci.nsIPrefService)
            .getBranch("browser.link.");
      var newWindow = linkprefs.getIntPref ("open_newwindow");

      if (newWindow == 2)
        recent.open(blog.URL);
      else {
        var browser = recent.document.getElementById('content');
        browser.loadOneTab(blog.URL, null, null, null, false);
      }
    },
    onError: function(aError) {
      $("accept-button").setAttribute('disabled', 'false');
      alert("Error publishing: "+aError.errorstring);
    },
    onFault: function(aError) {
      $("accept-button").setAttribute('disabled', 'false');
      alert("Error publishing: "+aError.errorstring);
    },
  }

  var service = Cc[blog.getParent().serviceId].getService(Ci.flockIBlogWebService);
  $("accept-button").setAttribute('disabled', 'true');
  service.newPost(listener, blog.id(), post, true, null);
}

function flock_blogEditorSetupEditorCommands() {
  if(gBlogEditCommandController) return;
  gBlogEditCommandController = new CommandController();

  gBlogEditCommandController.cmd_flock_blogLink = flock_blogLink;
  gBlogEditCommandController.cmd_flock_blogUnlink = flock_blogUnlink;

  window.controllers.appendController(gBlogEditCommandController);
  window.addEventListener('focus',function () { window.document.commandDispatcher.updateCommands('focus'); },false);
  window.addEventListener('click',function () { window.document.commandDispatcher.updateCommands('click'); },false);
//    window.addEventListener('dragdrop',function () { alert("drop"); },false);
  window.document.commandDispatcher.updateCommands('focus');
  window.focus();
}

function openBlogConfig() {
    window.openDialog("chrome://flock/content/blog/blogOptions.xul", "blogOptions", "chrome,centerscreen", null);
}

/*
 * Spellchecking
 */

function updateContextMenu(event) {
  InlineSpellCheckerUI.clearSuggestionsFromMenu();
  InlineSpellCheckerUI.clearDictionaryListFromMenu();

  InlineSpellCheckerUI.init(gEditor.mNSHTMLEditor);
  InlineSpellCheckerUI.initFromEvent(document.popupRangeParent, document.popupRangeOffset);

  var canSpell = InlineSpellCheckerUI.canSpellCheck;
  var onMisspelling = InlineSpellCheckerUI.overMisspelling;
  $("spell-check-enabled").hidden = !canSpell;
  // $("spell-separator").hidden = !(canSpell || this.possibleSpellChecking);
  $("spell-separator").hidden = canSpell;
  if (canSpell)
    document.getElementById("spell-check-enabled").setAttribute("checked", InlineSpellCheckerUI.enabled);
  $("spell-add-to-dictionary").hidden = !onMisspelling;

  // suggestion list
  $("spell-suggestions-separator").hidden = !onMisspelling;
  if (onMisspelling) {
      var menu = $("blogEditorContext");
      var suggestionsSeparator = document.getElementById("spell-add-to-dictionary");
      var numsug = InlineSpellCheckerUI.addSuggestionsToMenu (menu, suggestionsSeparator , 5);
      $("spell-no-suggestions").hidden = (numsug > 0);
  } else {
      $("spell-no-suggestions").hidden = true;
  }

  // dictionary list
  $("spell-dictionaries").hidden = !InlineSpellCheckerUI.enabled;
  if (canSpell) {
    var dictMenu = $("spell-dictionaries-menu");
    var dictSep = $("spell-language-separator");
    InlineSpellCheckerUI.addDictionaryListToMenu(dictMenu, dictSep);
  }
}

function formatURL(aFormat, aIsPref) {
  var formatter = Cc["@mozilla.org/toolkit/URLFormatterService;1"].getService(Ci.nsIURLFormatter);

  return aIsPref ? formatter.formatURLPref(aFormat) : formatter.formatURL(aFormat);
}

function addDictionaries() {
  var uri = formatURL("browser.dictionaries.download.url", true);

  var locale = "-";
  try {
    var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
    locale = prefService.getComplexValue("intl.accept_languages", Ci.nsIPrefLocalizedString).data;
  }
  catch (e) { }

  var version = "-";
  try {
    version = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULAppInfo).version;
  }
  catch (e) { }

  uri = uri.replace(/%LOCALE%/, escape(locale));
  uri = uri.replace(/%VERSION%/, version);

  var prefService = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch);
  var newWindowPref = prefService.getIntPref("browser.link.open_newwindow");
  var where = newWindowPref == 3 ? "tab" : "window";

  openUILinkIn(uri, where);
}


/*
 * Stuff that I don't know where to put
 */

function ljuser()
{
  var userid = prompt("Enter the LiveJournal ID of your friend", "");
  var html = '<lj user="'+userid+'">';
  $('edit').contentWindow.document.execCommand("inserthtml", false, html);
}

function execCommand(cmd)
{
  $('edit').contentWindow.document.execCommand(cmd, false, null);
}


/*
 * Sidebar
 */

// |forceOpen| is a bool that indicates that the sidebar should be forced open.  In other words
// the toggle won't be allowed to close the sidebar.
function toggleSidebar(aCommandID, forceOpen) {

  var sidebarBox = document.getElementById("sidebar-box");
  if (!aCommandID)
    aCommandID = sidebarBox.getAttribute("sidebarcommand");

  var elt = document.getElementById(aCommandID);
  var sidebar = document.getElementById("sidebar");
  var sidebarTitle = document.getElementById("sidebar-title");
  var sidebarSplitter = document.getElementById("sidebar-splitter");

  if (!forceOpen && elt.getAttribute("checked") == "true") {
    elt.removeAttribute("checked");
    sidebarBox.setAttribute("sidebarcommand", "");
    sidebarTitle.setAttribute("value", "");
    sidebarBox.hidden = true;
    sidebarSplitter.hidden = true;
    if (aCommandID != "flock_NewsSidebarBroadcaster")
      content.focus();
    return;
  }

  var elts = document.getElementsByAttribute("group", "sidebar");
  for (var i = 0; i < elts.length; ++i)
    elts[i].removeAttribute("checked");

  elt.setAttribute("checked", "true");;

  if (sidebarBox.hidden) {
    sidebarBox.hidden = false;
    sidebarSplitter.hidden = false;
  }

  var url = elt.getAttribute("sidebarurl");
  var title = elt.getAttribute("sidebartitle");
  if (!title)
    title = elt.getAttribute("label");
  sidebar.setAttribute("src", url);
  sidebarBox.setAttribute("src", url);
  sidebarBox.setAttribute("sidebarcommand", elt.id);
  sidebarTitle.setAttribute("value", title);
  if (aCommandID != "viewWebPanelsSidebar") { // no searchbox there
    // if the sidebar we want is already constructed, focus the searchbox
    if ((aCommandID == "viewBookmarksSidebar" && sidebar.contentDocument.getElementById("bookmarksPanel"))
    || (aCommandID == "viewHistorySidebar" && sidebar.contentDocument.getElementById("history-panel")))
      sidebar.contentDocument.getElementById("search-box").focus();
    // otherwiese, attach an onload handler
    else
      sidebar.addEventListener("load", asyncFocusSearchBox, true);
  }
}


var gDraftId = "";
var gLastSaved = "";
var gRecoveryId = "";

function bpflock_blogEditorSavePost() {
//return hasChanged();
   var html;
    if ($('editordeck').selectedIndex == 0)
      html = gEditor.getXHtml();
    else
      html = $('editsource').value;
    if (gDraftId == "")
      gDraftId = flock.blog.service.createDraft($("flock-blog-title").value, html, $('flock-blog-tags').value, false);
    else
      flock.blog.service.saveDraft(gDraftId, $("flock-blog-title").value, html, $('flock-blog-tags').value);
    updateLastSaved();
$('bpsavebutton').setAttribute('disabled',true);
}

function updateLastSaved() {
  gLastSaved = $("flock-blog-title").value + gEditor.getXHtml() + $('flock-blog-tags').value;
}

